<?php
// $Id

/**
 * @file
 * Page callback file for the blog module.
 */
 
/**
 * Pull in some helper functions, mainly for debugging.
 */
include_once 'helper_functions.php';
 
/**
 * Handles displaying a list of blogs in a couple of ways, either a striaght listing
 * of all blogs or a list of the blogs grouped by the user that created them
 */
function everyblog_list_blogs() {
	$output ='';
	
	// determine what order the blogs should be listed in
	if (isset($_GET['orderby'])) {
		$order_by = strtolower($_GET['orderby']);
	}
	if ( $order_by == 'recent') {
		$order_by == 'recent';
	}
	else {
		$order_by = 'alpha';
	}
	
	// The path might be "blogs/user" or "blogs/category" or just "blogs"
	if (func_num_args() == 1) {
		$view = func_get_arg(0);
		$view = split('\?', $view);
		$view = strtolower($view[0]);
	}
	else {
		$view = '';
	}
	
	// Show blogs grouped by user
	if( $view == t('user')) {
		$is_user_view = TRUE;
		list($title, $breadcrumb, $blogs, $error_message) = everyblog_get_blogs_grouped_by_user($order_by);
	}
	// Show blogs grouped by category
	else if ($view == t('category')) {
		$is_category_view = TRUE;
		list($title, $breadcrumb, $blogs, $error_message) = everyblog_get_blogs_grouped_by_category($order_by);
	}
	// Show a regular listing of all blogs
	else {
		$is_regular_view = TRUE;
		list($title, $breadcrumb, $blogs, $error_message) = everyblog_get_all_blogs($order_by);
	}
	
	// setup the title
	drupal_set_title($title);
	
	// Setup a basic breadcrumb
	drupal_set_breadcrumb(array(
		l(t('Home'), NULL), 
		l(t('Blogs'), 'blogs'),
		$breadcrumb));
		
	// all the user to sort the blogs
	switch ($order_by) {
		case 'recent':
			$output .= '['.l(t('Order Alphabetically'), t('blogs/!view', array('!view'=>$view)), array('query'=>'orderby=alpha')).']';
			break;
		
		case 'alpha':		
		default:
			$output .= '['.l(t('Order by Most Recent Post'), t('blogs/!view', array('!view'=>$view)), array('query'=>'orderby=recent')).']';
	}
	$output .= "<br>\n";
		
	// display the blogs
	if (count($blogs) > 0) {
		if ($is_regular_view) {
			$output .= theme('item_list', $blogs);
		}
		elseif ($is_user_view || $is_category_view) {
			foreach ($blogs as $groupname=>$group) {
				$output .= $groupname;
				$output .= theme('item_list', $group);
			}
		}
	}
	else {
		drupal_set_message($error_message, 'status');
	}

	// give the user an option to see the blogs grouped by user or by category
	if ($is_regular_view) {		
		$output .= '<br>';
		$output .= l(t('View Blogs Grouped by User'), t('blogs') . '/' . t('user'));
		$output .= '<br>';
		$output .= l(t('View Blogs Grouped by Category'), t('blogs') .'/' . t('category'));
	}
			
	drupal_add_feed(url(t('blogs') . '/' . t('feed')), t('RSS - All Blogs'));
	
	return $output;
}

function everyblog_get_all_blogs($order_by) {
	$title = t('All Blogs');
	$breadcrumb = t('All Blogs');
	
	if ($order_by == 'alpha') {
		$query = <<<QUERY
SELECT 
	n.nid AS blog_id, 
	n.title AS blog_name 
FROM 
	{node} AS n 
WHERE 
	n.type = 'everyblog_blog' 
ORDER BY 
	blog_name
QUERY;
	}
	elseif ($order_by == 'recent') {
		$query = <<<QUERY
SELECT * FROM (
	SELECT 
		b.title AS blog_name, 
		n.created 
	FROM 
		{node} AS b 
	JOIN {everyblog_entries} AS e 
		ON (b.nid = e.blog_id) 
	JOIN node AS n 
		ON (e.nid = n.nid) 
	GROUP BY blog_name 
	ORDER BY n.created DESC
) AS blogs ORDER BY created DESC
QUERY;
	}

	$result = db_query($query);
	while($blog = db_fetch_object($result)) {
		$blogs[] = l($blog->blog_name, t('blog') . '/' . _everyblog_encode($blog->blog_name))
			. (variable_get('everyblog_show_datetime', FALSE) ?
				' (<i>' . date('D, M j, Y G:i', $blog->created) . '</i>)' : '');
	}
	return array($title, $breadcrumb, $blogs, $error_message);
}

function everyblog_get_blogs_grouped_by_category($order_by) {
	$title = t('All Blogs Grouped by Category');
	$breadcrumb = t('By Category');
	 
	// Get a list of categories blogs are in
	if ($order_by == 'alpha') {
	}
	elseif ($order_by == 'recent') {
	}
	
	// get a list of vocabularies that can be applied to everyblog_blog nodes
	$vocabs = taxonomy_get_vocabularies('everyblog_blog');
	foreach ($vocabs as $vocab) {
		
		// get a list of terms associated with the vocabulary
		$terms =  taxonomy_get_tree($vocab->vid);
		foreach ($terms as $term) {
			// encode urls for the vocab and term
			$vocaburl = _everyblog_encode($vocab->name);
			$termurl = _everyblog_encode($term->name);
			
			$group = l($term->name, t('blogs') . "/$vocaburl/$termurl");
			
			// determine how we are going to order things
			if ($order_by == 'alpha') {
				$order_by = 'n.sticky DESC, n.title ASC';
			}
			elseif ($order_by == 'recent') {
				$order_by = 'n.sticky DESC, n.created DESC';
			}
			$result = taxonomy_select_nodes(array($term->tid), $order_by);
			
			// list all the blogs
			while ($blog = db_fetch_object($result)) {
				$blogs[$group][] = l($blog->title, t('blog') . '/' . _everyblog_encode($blog->title));
			}

			$error_message = t('There are no blogs in this category yet.');
		}
	}
	
	return array($title, $breadcrumb, $blogs, $error_message);
}

function everyblog_get_blogs_grouped_by_user($order_by) {
	$title = t('All Blogs Grouped By User');
	$breadcrumb = t('By User');
	
	// Get a list of users who have made blogs
	if ($order_by == 'alpha') {
		$query = <<<QUERY
SELECT 
	n.uid
FROM 
	{node} AS n 
JOIN {users} AS u ON (n.uid = u.uid)
WHERE 
	n.type = 'everyblog_blog' 
GROUP BY uid ORDER BY `name`
QUERY;
	}
	elseif ($order_by == 'recent') {
		$query = <<<QUERY
SELECT * FROM (
	SELECT 
		b.title AS blog_name, 
		b.uid,
		n.created 
	FROM 
		{node} AS b 
	JOIN {everyblog_entries} AS e 
		ON (b.nid = e.blog_id) 
	JOIN node AS n 
		ON (e.nid = n.nid) 
	GROUP BY b.uid 
	ORDER BY n.created DESC
) AS blogs ORDER BY created DESC
QUERY;
		#'SELECT * FROM (SELECT {everyblog_blogs}.uid, created FROM {everyblog_blogs} JOIN ({everyblog_entries}) USING (blog_id) JOIN ({node}) USING (nid) ORDER BY created DESC) AS t GROUP BY uid ORDER BY created DESC');
	}
	
	// For each user who has blogs
	$result = db_query($query);
	while ($u = db_fetch_object($result)) {
		// load the user so we can get their username
		$user = user_load($u->uid);
		$group = l($user->name, t('blogs') . '/' . t('user') . '/' . $user->name);
		
		// get a list of blogs that the user has
		if ($order_by == 'alpha') {
			$query = "SELECT n.nid AS blog_id, n.title AS blog_name FROM {node} AS n WHERE n.type='everyblog_blog' AND n.uid=%s ORDER BY n.title";
		}
		elseif ($order_by == 'recent') {
			$query = <<<QUERY
SELECT * FROM (
	SELECT 
		b.title AS blog_name, 
		n.created 
	FROM 
		{node} AS b 
	JOIN {everyblog_entries} AS e 
		ON (b.nid = e.blog_id) 
	JOIN node AS n 
		ON (e.nid = n.nid) 
	WHERE 
		b.uid = %s
	GROUP BY blog_name 
	ORDER BY n.created DESC
) AS blogs ORDER BY created DESC
QUERY;
		}
		
		$result = db_query($query, $user->uid);
		while ($blog = db_fetch_object($result)) {
			$blogs[$group][] = l($blog->blog_name, t('blog') . '/' . _everyblog_encode($blog->blog_name))
				. (variable_get('everyblog_show_datetime', FALSE) ?
					' (<i>' . date('D, M j, Y G:i', $blog->created) . '</i>)' : '');
		}
		
		$error_message = t('There are no blogs by this user yet.');
	}
	
	return array($title, $breadcrumb, $blogs, $error_message);
}

/**
 * Lists all the blogs by a particular user
 *
 * @param $uid mixed if its a number its assumed to be a uid otherwise its taken to be the username
 */
function everyblog_list_blogs_by_user($user) {
	$output ='';
	
	if (is_null($user)) {
		drupal_set_message(t('No matching user found. Please check for typing errors in the URL.'), 'error');
		return '';
	}
	
	$vocab_url = _everyblog_encode($vocab->name);
	$term_url = _everyblog_encode($term->name);	

	$uid = $user->uid;
	drupal_set_title($title=t('Blogs by ') . $user->name);
	drupal_set_breadcrumb(array(
		l(t('Home'), NULL),
		l(t('Blogs'), t('blogs')),
		l(t('User'), t('blogs') . '/' . t('user')),
		$user->name,
		));
		
	// determine what order the blogs should be listed in
	if (isset($_GET['orderby'])) {
		$order_by = strtolower($_GET['orderby']);
	}
	else {
		$order_by = 'alpha';
	}
	
	// determine how we are going to order things
	if ($order_by == 'alpha') {
		$order_by = 'alpha';
		$output .= '['.l(t('Order by Most Recent Post'), "blogs/$vocab_url/$term_url", array('query'=>'orderby=recent')).']';
		$order_by = 'n.sticky DESC, n.title ASC';
	}
	elseif ($order_by == 'recent') {
		$order_by == 'recent';
		$output .= '['.l(t('Order Alphabetically'), "blogs/$vocab_url/$term_url", array('query'=>'orderby=alpha')).']';
		$order_by = 'n.sticky DESC, n.created DESC';
	}
	$output .= "<br>\n";
		
	$query = 'SELECT n.nid AS blog_id, n.title AS blog_name FROM {node} AS n WHERE n.status = 1 AND n.type = \'everyblog_blog\' AND n.uid='.$uid.' ORDER BY blog_name';
	$result = db_query($query);
	while($blog = db_fetch_object($result)) {
		$blogs[] = l($blog->blog_name, t('blog') . '/' . _everyblog_encode($blog->blog_name))
			. (variable_get('everyblog_show_datetime', FALSE) ?
				' (<i>' . date('D, M j, Y G:i', $blog->created) . '</i>)' : '');
	}
	$output .= theme('item_list', $blogs);
	
	drupal_add_feed(url(t('blogs') . '/' . t('user') . '/'. $user->name . '/' . t('feed')), t('RSS - Blogs by !username', array('!username' => $user->name)));
	return $output;	
}

/**
 * Lists all the blogs by a particular user
 *
 * @param $uid mixed if its a number its assumed to be a uid otherwise its taken to be the username
 */
function everyblog_list_blogs_by_category($vocab, $term) {
	$output ='';
	
	// If the URL part is numeric then its a taxonomy term id
	if (is_null($term)) {		
		drupal_set_message(t('No matching category was found. Please check for typing errors in the URL.'), 'error');			
		return '';
	}
	
	$vocab_url = _everyblog_encode($vocab->name);
	$term_url = _everyblog_encode($term->name);
	
	drupal_set_title($title=t('Blogs in ') . $term->name);
	drupal_set_breadcrumb(array(
		l(t('Home'), NULL),
		l(t('Blogs'), t('blogs')),
		l($vocab->name, t('blogs') . '/' . $vocab_url),
		$term->name,
		));

	// determine what order the blogs should be listed in
	if (isset($_GET['orderby'])) {
		$order_by = strtolower($_GET['orderby']);
	}
	else {
		$order_by = 'alpha';
	}
	
	// determine how we are going to order things
	if ($order_by == 'alpha') {
		$order_by = 'alpha';
		$output .= '['.l(t('Order by Most Recent Post'), "blogs/$vocab_url/$term_url", array('query'=>'orderby=recent')).']';
		$order_by = 'n.sticky DESC, n.title ASC';
	}
	elseif ($order_by == 'recent') {
		$order_by == 'recent';
		$output .= '['.l(t('Order Alphabetically'), "blogs/$vocab_url/$term_url", array('query'=>'orderby=alpha')).']';
		$order_by = 'n.sticky DESC, n.created DESC';
	}
	$output .= "<br>\n";
	
	$result = taxonomy_select_nodes(array($term->tid), $order_by);
	
	// list all the blogs
	while ($blog = db_fetch_object($result)) {
		$blogs[] = l($blog->title, t('blog') . '/' . _everyblog_encode($blog->title))
			. (variable_get('everyblog_show_datetime', FALSE) ?
				' (<i>' . date('D, M j, Y G:i', $blog->created) . '</i>)' : '');
	}		
	
	$output .= theme('item_list', $blogs);
	
	drupal_add_feed(url(t('blogs') . '/' . t('category') . '/' . $category->name . '/' . t('feed')), 
		t('RSS - Blogs in !category', array('!category' => $category->name)));
	return $output;
}

/**
 * Displays recent blog entries of a given blog.
 *
 * @param $blog_id mixed if its a number then its taken to be the blogs id, otherwise its
 *		taken to be the blogs name, either URL encoded or with spaces replaced by
 * 		underscores '_'
 */
function everyblog_page($blog) {
	return node_view(node_load($blog->blog_id), FALSE, TRUE, FALSE);
}

function everyblog_nodeapi(&$node, $op, $a3, $a4) {
	global $user;
	
	if ($op == 'alter') {
		if ($node->type == 'everyblog' && $user->uid == $node->uid && $node->status == '0') {
			$node->title = t('[Unpublished]') . ' ' . $node->title;
		}
	}
}

/**
 * Menu callback; displays a Drupal page containing recent blog entries of all users.
 */
function everyblog_page_last() {
	global $user;

	$output = '';
	$items = array();

	if (user_access(t('edit own blog'))) {
		$items[] = l(t('Create new blog entry.'), "node/add/everyblog");
	}

	$output = theme('item_list', $items);

	$result = pager_query(db_rewrite_sql("SELECT n.nid, n.created FROM {node} n WHERE n.type = 'blog' AND n.status = 1 ORDER BY n.sticky DESC, n.created DESC"), variable_get('default_nodes_main', 10));
	$has_posts = FALSE;

	while ($node = db_fetch_object($result)) {
		$output .= node_view(node_load($node->nid), 1);
		$has_posts = TRUE;
	}
	
	if ($has_posts) {
		$output .= theme('pager', NULL, variable_get('default_nodes_main', 10));
	}
	else {
		drupal_set_message(t('No blog entries have been created.'));
	}
	drupal_add_feed(url('blog/feed'), t('RSS - blogs'));

	return $output;
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries of a given user.
 */
function everyblog_feed_user($uid) {
	// load the user
	if (is_numeric($uid)) {
		$user = user_load($uid);
	}
	else {
		$user = user_load(array('name'=>$uid));
	}
	if ($user == FALSE) {
		$channel['title'] = t('No blogs by @username', array('@username' => $uid));
		$channel['link'] = url("/", array('absolute' => TRUE));
		$items = array();
		node_feed($items, $channel);
		return;
	}
	$uid = $user->uid;

	// Set the channel information
	$channel['title'] = t('Blogs by @username', array('@username' => $user->name));
	$channel['link'] = url(t('blogs') . '/' . t('user') . '/' . $uid, array('absolute' => TRUE));
	
	$query = <<<EOF
SELECT e.blog_id, e.blog_name, n.nid, n.created 
FROM {everyblog_blogs} AS e, {everyblog_entries} AS etn, {node} AS n 
WHERE e.uid=$uid AND e.blog_id = etn.blog_id AND etn.nid = n.nid AND n.status = 1 ORDER BY n.created DESC
EOF;
	
	// Run the query 
	$result = db_query_range($query, 0, variable_get('feed_default_items', 10));

	// get the feed items
	while($entry = db_fetch_object($result)) {
		$entries[] = $entry->nid;
	}
	node_feed($entries, $channel);
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries in a given category
 */
function everyblog_feed_category($tid) {
	// If the URL part is numeric then its a taxonomy term id
	if (is_numeric($tid)) {		
		$category = taxonomy_get_term($tid);
	}
	// The URL part is a category name, so look it up
	else {
		$category = taxonomy_get_term_by_name($tid);
		$category = $category[0];
	}
	if ($category === FALSE) {
		$channel['title'] = t('No category @tid', array('@tid'=>$tid));
		$channel['link'] = url("/", array('absolute' => TRUE));
		$items = array();
		node_feed($items, $channel);
		return;
	}
	$tid = $category->tid;

	// Set the channel information
	$channel['title'] = t('Blogs in @category', array('@category' => $category->name));
	$channel['link'] = url(t('blogs') . '/' . t('category') . '/' . $tid, array('absolute' => TRUE));
	
	$query = <<<EOF
SELECT e.blog_id, e.blog_name, n.nid, n.created 
FROM {everyblog_blogs} AS e, {everyblog_entries} AS etn, {node} AS n 
WHERE e.tid=$tid AND e.blog_id = etn.blog_id AND etn.nid = n.nid AND n.status = 1 ORDER BY n.created DESC
EOF;
	
	// Run the query 
	$result = db_query_range($query, 0, variable_get('feed_default_items', 10));

	// get the feed items
	while($entry = db_fetch_object($result)) {
		$entries[] = $entry->nid;
	}
	node_feed($entries, $channel);
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries in a given blog.
 */
function everyblog_feed_blog($blog_id) {
	$blog = everyblog_blog_loar($blog_id);

	// Set the channel information
	$channel['title'] = t('@blogname: Recent Entries', array('@blogname' => $blog->blog_name));
	$channel['link'] = url(t('blog'). '/' . $blog->title, array('absolute' => TRUE));
	
	$query = <<<EOF
SELECT e.blog_id, e.blog_name, n.nid, n.created 
FROM {everyblog_blogs} AS e, {everyblog_entries} AS etn, {node} AS n 
WHERE e.blog_id=$blog_id AND e.blog_id = etn.blog_id AND etn.nid = n.nid AND n.status = 1 ORDER BY n.created DESC
EOF;
	
	// Run the query 
	$result = db_query_range($query, 0, variable_get('feed_default_items', 10));

	// get the feed items
	while($entry = db_fetch_object($result)) {
		$entries[] = $entry->nid;
	}
	node_feed($entries, $channel);
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries of all blogs.
 */
function everyblog_feed_all() {
	$result = db_query_range(db_rewrite_sql("SELECT n.nid, n.created FROM {node} n WHERE n.type = 'everyblog' AND n.status = 1 ORDER BY n.created DESC"), 0, variable_get('feed_default_items', 10));
	$channel['title'] = variable_get('site_name', 'Drupal') .' blogs';
	$channel['link'] = url(t('blog'), array('absolute' => TRUE));

	$items = array();
	while ($row = db_fetch_object($result)) {
		$items[] = $row->nid;
	}

	node_feed($items, $channel);}