<?php 
    
class EveryBlogBaseTestCase extends DrupalWebTestCase { 
	/**
	 * A basic user who can only create posts in their own blogs
	 */
	var $basic_user;
	
	/**
	 * Another user who just logs in looking at blogs
	 */
	var $other_user;
	
	/**
	 * A power user that can post to any blog
	 */
	 var $power_user;
	 
	 /**
	  * Array for holding blogs by the users
	  */
	 var $blogs = array();
	 
	 /* Permissions List:
	  * 'create own blog', 
	  * 'edit own blog',
	  * 'delete own blog',
	  * 'create own blog entry', 
	  * 'edit own blog entry', 
	  * 'delete own blog entry',
	  * 
	  * 'edit any blog',
	  * 'delete any blog',
	  * 'create any blog entry', 
	  * 'edit any blog entry',
	  * 'delete any blog entry', 
	  */
	 function setUp() {
		parent::setUp('everyblog');	
	 
		// Create a basic user
		$permissions = array(
			'access content',
			'create story content',
			'create own blog',
			'edit own blog',
			'delete own blog',
			'create own blog entry',
			'edit own blog entry',
			'delete own blog entry',
			);
		$basic_user = $this->drupalCreateUser($permissions);
		$basic_user->simplename = 'basic_user';
		
		// Create an other user, that can't create blogs
		$permissions = array(
			'access content'
			);
		$other_user = $this->drupalCreateUser($permissions);
		$other_user->simplename = 'other_user';

		// Create a power user
		$permissions = array(
			'access content',
			'access administration pages',
			'administer site configuration',
			'administer taxonomy',
			
			'create own blog',
			'edit own blog',
			'delete own blog',
			'create own blog entry',
			'edit own blog entry',
			'delete own blog entry',
			
			'edit any blog',
			'delete any blog',
			'create any blog entry',
			'edit any blog entry',
			'delete any blog entry',
			);
		$power_user = $this->drupalCreateUser($permissions);
		$power_user->simplename = 'power_user';
		
		// Assign users to their test suite-wide properties.
		$this->basic_user = $basic_user;
		$this->other_user = $other_user;
		$this->power_user = $power_user;
	 }
	 
	function tearDown() {
		parent::tearDown();
		
		unset($this->blogs);
	}
	
	function _createBlog($user, $vocab=NULL, $term=NULL) {
		// Login the given user
		$this->drupalLogin($user);
		$uname = $user->simplename;
		
		// Setup the array
		if (!isset($this->blogs[$uname])) {
			$this->blogs[$uname] = array();
		}
		
		// set the title and the body of the new blog
		$edit = array(
			'title' => 'Blog ' . $this->randomName(32),
			'body' => $this->randomName(32),
			);
		
		// if there was a vocabulary and a tid given
		if (!is_null($vocab) && !is_null($vocab)) {
			// associate the blog with the term
			$edit["taxonomy[{$vocab->vid}]"] = $term->tid;
		}
		
		// create the blog
		$this->drupalPost('node/add/everyblog-blog', $edit, t('Save'));
		
		// store the newly created blog
		$i = count($this->blogs[$uname]);
		$this->blogs[$uname][$i] = node_load(array('title' => $edit['title']));
		$this->assertNotNull($this->blogs[$uname][$i], "Created blog $i for $uname nid=" . $this->blogs[$uname][$i]->nid);

		$this->drupalLogout();
		
		return $this->blogs[$uname][$i];
	}
	
	function _editBlog($nid, $msg) {
		$edit = array(
			'title' => 'Blog ' . $this->randomName(32),
			'body' => $this->randomName(32),
			);
		
		$this->drupalPost("node/$nid/edit", $edit, t('Save'));
		
		// check the edit
		$this->drupalGet("node/$nid");
		$this->assertText($edit['title'], $msg.': Title changed');
		$this->assertText($edit['body'], $msg.': Body changed');
	}
	
	function _createBlogEntry($blog, $user_name) {
	 	if (is_numeric($blog)) {
	 		$blog_id = $blog;
	 	}
	 	else {
	 		$blog_id = $blog->nid;	 		
	 	}
	 	
	 	$edit = array(
	 		'title' => 'Entry ' . $this->randomName(32),
	 		'body' => $this->randomName(32),
	 		'blog_id' => $blog_id,
	 		);
	 	$this->drupalPost("node/add/everyblog-entry", $edit, t('Save'));
	 	$entry = node_load(array('title' => $edit['title']));
	 	
	 	$this->assertText(t('Blog entry @title has been created.', array('@title'=>$edit['title'])), 
	 		"$user_name created blog entry [$blog_id].");
	 		
	 	return $entry;
	}
	 
	function _editEntry($entry, $uname) {
 	 	if (is_numeric($entry)){
	 		$entry = noad_load($entry);
	 	}
	 	
	 	$nid = $entry->nid;
	 	
	 	$edit = array(
			'title' => 'Entry ' . $this->randomName(32),
			'body' => $this->randomName(32),
			);
		
		$this->drupalPost("node/$nid/edit", $edit, t('Save'));
		
		// check the edit
		$this->drupalGet("node/$nid");
		$this->assertText($edit['title'], $uname . ': Title changed');
		$this->assertText($edit['body'], $uname . ': Body changed'); 
	}	
}
?>